#include <stdio.h>
#include <mpi.h>
#include <stdlib.h>

#define MAX 15

void initializeArray(int *array, int size)
{
	for (int i = 0; i < size; ++i)
		array[i] = 1+rand()%10;
}

void printArray(int *array, int size)
{
	for (int i = 0; i < size; ++i)
		printf("%d ", array[i]);
	printf("\n");
}

int computeSum(int *array, int start, int end)
{
	int sum = 0;
	for (int i = start; i <= end; ++i)
		sum = sum + array[i];
	return sum;
}

void main (int argc, char *argv[]) {
	
	MPI_Status status;
	int myrank, size, retVal;	
	int sumMaster = 0;
	int sumSlave = 0;
	int totalSumSlave = 0;
	int q = 0, r = 0;
	
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	q = MAX / size;
	r = MAX % size;

	if (myrank == 0)
	{
		// I'm the master
		int buf[MAX];
		initializeArray(buf, MAX);
		printArray(buf, MAX);
		// master sends only a section of the original vector to all slaves
		for (int p = 1; p < size; ++p)
			retVal = MPI_Send(&buf[q*p], q, MPI_INT, p, 555, MPI_COMM_WORLD);
		// master computes the sum operation of the vector's head
		sumMaster = computeSum(buf, (myrank*q), ((myrank+1)*q)-1);
		// master computes the sum operation of the vectorìs tail
		sumMaster = sumMaster + computeSum(buf, MAX-r, MAX-1); 
		// master receives from all workers their results, them prints the sum of summing
		for (int p = 1; p < size; ++p)
		{
			MPI_Recv(&sumSlave, 1, MPI_INT, p, 555, MPI_COMM_WORLD, &status);
			totalSumSlave = totalSumSlave + sumSlave;
		}
		printf("La somma degli elementi dell'array è %d\n", sumMaster+totalSumSlave);
	}
	else
	{
		//I'm the slave
		int buf[MAX];
		// slaves receive a small vector...
		retVal = MPI_Recv(buf, q, MPI_INT, 0, 555, MPI_COMM_WORLD, &status);
		// ... then compute 
		sumSlave = computeSum(buf, 0, q-1);
		// ... and sends back the results to the master
		retVal = MPI_Send(&sumSlave, 1, MPI_INT, 0, 555, MPI_COMM_WORLD);
	}
	
	MPI_Finalize();
}





