#include <stdio.h>
#include <mpi.h>
#include <stdlib.h>

#define MAX 21

void initializeArray(int *array, int size)
{
	for (int i = 0; i < size; ++i)
		array[i] = 1+rand()%100;
}

void printArray(int *array, int size)
{
	for (int i = 0; i < size; ++i)
		printf("%d ", array[i]);
	printf("\n");
}

void swap(int array[], int i, int j)
{
  int temp;
  temp = array[i];
  array[i] = array[j];
  array[j] = temp;
}

void merge(int first[], int sizeFirst, int second[], int sizeSecond, int array[])
{
	int size = sizeFirst+sizeSecond; 
	//int array[size];
	int i = 0, f = 0, s = 0;

	while (i < size && f < sizeFirst && s < sizeSecond)
	{
		if (first[f] < second[s])
		{
			array[i] = first[f];
			f = f+1;
		}
		else
		{
			array[i] = second[s];
			s = s+1;
		}
		i = i+1;
	}

	if (f == sizeFirst)
	{
		while (s < sizeSecond)
		{
			array[i] = second[s];
			s = s+1;
			i = i+1;
		}
	}
	else
	{
		while (f < sizeFirst)
		{
			array[i] = first[f];
			f = f+1;
			i = i+1;
		}
	}
	//return array;
}

void copy(int source[], int dest[], int size)
{
	for (int i = 0; i < size; ++i)
		dest[i] = source[i];
}

void sort(int array[], int size)
{
  int i, j;
  for (i = 0; i < size-1; ++i)
  {
    int min = i;
    for (j = i + 1; j < size; ++j)
    {
      if (array[j] < array[min])
	min = j;
    }
    swap(array, i, min);
  }
}

void main (int argc, char *argv[])
{
	MPI_Status status;
	int myrank, size, retVal;
	int q = 0, r = 0;
	
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	q = MAX / size;
	r = MAX % size;

	if (myrank == 0)
	{
		int array[MAX];
		int sortedArray[MAX];
		int tmp[MAX];
		int arrayMaster[q+r];
		int sortedArraySlave[q];
		

		initializeArray(array, MAX);
		printArray(array, MAX);
		// Only a small section of the vector is sent
		for (int p = 1; p < size; ++p)
			retVal = MPI_Send(&array[q*p+r], q, MPI_INT, p, 555, MPI_COMM_WORLD);
		
		// master takes a subarray (having size Q+R) and sorts it
		copy(array, arrayMaster, q+r);
		sort(arrayMaster, q+r);		// arrayMaster is now sorted
		copy(arrayMaster, sortedArray, q+r);
		
		for (int p = 1; p < size; ++p)
		{
			// Master receives from all slaves an ordered subarray (having size Q)
			MPI_Recv(sortedArraySlave, q, MPI_INT, p, 555, MPI_COMM_WORLD, &status);
			copy(sortedArray, tmp, (p+1)*q+r);
			// merging operation
			merge(tmp, (p+1)*q+r, sortedArraySlave, q, sortedArray);
		}
		printArray(sortedArray, MAX);
	}
	else
	{
		int arraySlave[q];
		// Slave receives a small vector
		retVal = MPI_Recv(arraySlave, q, MPI_INT, 0, 555, MPI_COMM_WORLD, &status);
		sort(arraySlave, q);
		// Sending back sorted vector
		retVal = MPI_Send(arraySlave, q, MPI_INT, 0, 555, MPI_COMM_WORLD);
	}

	MPI_Finalize();
}
