#include <stdio.h>
#include <mpi.h>
#include <stdlib.h>

#define N 4

void initializeMatrix(int rows, int cols, int matrix[rows][cols])
{
	for (int i = 0; i < rows; ++i)
		for (int j = 0; j < cols; ++j)
			matrix[i][j] = rand() % 20;
}

void zero(int rows, int cols, int matrix[rows][cols])
{
	for (int i = 0; i < rows; ++i)
		for (int j = 0; j < cols; ++j)
			matrix[i][j] = 0;
}

void initializeVector(int rows, int vector[rows])
{
	for (int i = 0; i < rows; ++i)
		vector[i] = rand() % 20;
}


void printMatrix(int rows, int cols, int matrix[rows][cols])
{
	for (int i = 0; i < rows; ++i)
	{
		for (int j = 0; j < cols; ++j)
			printf("%2d ", matrix[i][j]);
		printf("\n");
	}
}

void printVector(int rows, int vector[rows])
{
	for (int j = 0; j < rows; ++j)
		printf("%2d\n", vector[j]);
}

void mult(int rows, int cols, int matrix[rows][cols], int vector[cols], int result[rows])
{
	for (int i = 0; i < rows; ++i)
	{
		result[i] = 0;
		for (int j = 0; j < cols; ++j)
			result[i] = result[i] + matrix[i][j]*vector[j];
	}
}

void main (int argc, char *argv[])
{
	MPI_Status status;
	int myrank, P;

	// To keep algorithm simple, we fix to 2, 4 or 8 the number of processes
	// and 8, 16 or 32 the size of the square matrix and the vector
	// where 8 is a multiple of 4

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
	MPI_Comm_size(MPI_COMM_WORLD, &P);

	int sendMatrix[N][N];
	int recvMatrix[N/P][N];
	int vector[N];
	int localResult[N/P];
	int result[N];

	if (myrank == 0)
	{
		// STEP 1: process 0 inizializes matrix and vector, then prints
		initializeMatrix(N, N, sendMatrix);
		initializeVector(N, vector);

		// print sendMatrix and vector
		printf("Matrix:\n");
		printMatrix(N, N, sendMatrix);
		printf("\n");
		printf("Vector:\n");
		printVector(N, vector);
	}
	
	// STEP 2: Process 0 scatters matrix to all processes
	// of course not the whole matrix is sent to all workers, only a submatrix
	MPI_Scatter(sendMatrix, N*N/P, MPI_INT, 
		    recvMatrix, N*N/P, MPI_INT,
                    0, MPI_COMM_WORLD);

	// STEP 3: Process 0 broadcasts vector to all processes
	MPI_Bcast(vector, N, MPI_INT, 0, MPI_COMM_WORLD);

	// STEP 4: each worker computes multiplication,
	// then stores the results in a local vector
	mult(N/P, N, recvMatrix, vector, localResult);

	// STEP 5: process 0 gathers all results
	MPI_Gather(localResult, N/P, MPI_INT, result, N/P, MPI_INT, 0, MPI_COMM_WORLD);
	
	// STEP 6: master prints the result
	if (myrank == 0) 
	{
		printf("\nResult:\n");
		printVector(N, result);
	}

	MPI_Finalize();
}
