#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#define N 16

void initializeArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                array[i] = 1+rand()%10;
}

void printArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                printf("%d ", array[i]);
        printf("\n");
}

int computeSum(int *array, int start, int end)
{
        int sum = 0;
        for (int i = start; i <= end; ++i)
                sum = sum + array[i];
        return sum;
}


int main(int argc, char* argv[])
{
    MPI_Init(&argc, &argv);
 
    // Get my rank
    int my_rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    // Get number of processes and check that the buffer size can be splitted among all processes
    int size;
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    if (N % size != 0)
    {
	    if (my_rank == 0) printf("The number %d of elements in the array cannot be splitted among all %d processes\n", N, size);
	    MPI_Finalize();
	    return 0;
    }

    int buffer_to_send[N];	// buffer used by the master to scatter data
    int buffer_to_recv[N/size];	// smaller buffer used by the worker
 
    if(my_rank == 0)
    {
	initializeArray(buffer_to_send, N);
        printf("Values to scatter from process 0:\n");
	printArray(buffer_to_send, N);
    }
    // Scatter data to all processes: it sends N/size elements to each worker
    MPI_Scatter(buffer_to_send, N/size, MPI_INT, buffer_to_recv, N/size, MPI_INT, 0, MPI_COMM_WORLD);
    // all processes compute their data and show the result
    int sum = computeSum(buffer_to_recv, 0, N/size-1);
    printf("Process %d: %d\n", my_rank, sum);
 
    MPI_Finalize();
 
    return EXIT_SUCCESS;
}

