#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#define N 4

void initializeArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                array[i] = 1+rand()%10;
}

void printArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                printf("%d ", array[i]);
        printf("\n");
}

int main(int argc, char* argv[])
{
    MPI_Init(&argc, &argv);
 
    // Get my rank
    int my_rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    int size;
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    int buffer_to_send[N];	// buffer used by all workers to send data
    int buffer_to_recv[N];	// buffer used by all workers to receive data
 
    // all workers initialize their own small vector
    initializeArray(buffer_to_send, N);
    
    // all process wants to send only N/size elements from the others,
    // all process wants to receive only N/size elements from the others
    MPI_Alltoall(buffer_to_send, N/size, MPI_INT, buffer_to_recv, N/size, MPI_INT, MPI_COMM_WORLD);

    // Only master prints the original data, then the received data
    if (my_rank == 0) {
	printf("Original vector\n"); printArray(buffer_to_send, N);
	printf("Received vector\n"); printArray(buffer_to_recv, N);
    }
 
    MPI_Finalize();
 
    return EXIT_SUCCESS;
}

