#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

#define N 8

void initializeArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                array[i] = 1+rand()%10;
}

void printArray(int *array, int size)
{
        for (int i = 0; i < size; ++i)
                printf("%d ", array[i]);
        printf("\n");
}

int main(int argc, char* argv[])
{
    MPI_Init(&argc, &argv);
 
    // Get my rank
    int my_rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);

    int size;
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    int buffer_to_send[N];	// buffer used by all workers to send data
    int buffer_to_recv[N/size];	// buffer retrieved by the master
    int recvcount[size];	// number of element sent to others

    // initialize the vector with the number of element each process should receive
    for (int i = 0; i < size; ++i) recvcount[i] = N/size; // all processes will receive N/size elements
 
    // all workers initialize their own small vector
    initializeArray(buffer_to_send, N);
    
    // data are reduced and then scattered
    MPI_Reduce_scatter(buffer_to_send, buffer_to_recv, recvcount, MPI_INT, MPI_SUM, MPI_COMM_WORLD);

    // Only master prints the original data, then the received data
    if (my_rank == 0) {
	printf("Results\n"); printArray(buffer_to_recv, N/size);
    }
 
    MPI_Finalize();
 
    return EXIT_SUCCESS;
}

